==> 2.0.4.tar.gz: lib/listener.dart <==
import 'dart:async';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'variables.dart';
import 'api.dart';
import 'dart:developer';
import 'function.dart';

class Event {
  final EventType _event;
  final Node? _old;
  final Node? _new;

  Event(this._event, this._old, this._new);

  EventType get type => _event;

  Node? get oldNode => _old;

  Node? get newNode => _new;

  @override
  String toString() {
    return '${type.toString()} ${oldNode.toString()}=>${newNode.toString()}';
  }
}

class SensorValueListener implements StorageListener {
  List<Event> events = <Event>[];

  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) async {
    Event e = Event(event, oldNode, newNode);

    log('============================got event ${e.newNode!.path}');

    if(newNode!.path.contains(':data:metrics') && newNode.name != 'metrics'){

      events.add(e);
      Storage.getValuePath.add(newNode.parentPath);
      await updateData(events.iterator);
      events.remove(e);
    }else if(newNode.path.contains(':data:GeigerScoreUser')){
      events.add(e);
      var str = await newNode.getValue('implementedRecommendations');
      log('geiger-indicator - new IR: '+ str.toString());
      log('geiger-indicator - old IR: '+ CoreValues.userImplementedRecommendations.toString());

      if(str.toString() != CoreValues.userImplementedRecommendations.toString()) {
        generateImplementedReco(str.toString(),Types.users);
        await updateScoreNode(Types.users, Types.geigerScoreUser);
      }
      events.remove(e);

    }else if(newNode.path.contains(':data:GeigerScoreDevice') ){

      events.add(e);
      var str = await newNode.getValue('implementedRecommendations');
      log('geiger-indicator - new IR: '+ str.toString());
      log('geiger-indicator - old IR: '+ CoreValues.userImplementedRecommendations.toString());

      if(str.toString() != CoreValues.deviceImplementedRecommendations.toString()){
        generateImplementedReco(str.toString(),Types.devices);
        await updateScoreNode(Types.devices, Types.geigerScoreDevice);
      }
      events.remove(e);
    }

    //if(newNode?.name != 'metrics') await updateData(newNode!);

  }

  Future<List<Event>> getNumEvents(int num, [int timeout = 2000]) async {
    int start = DateTime.now().millisecondsSinceEpoch;
    List<Event> ret = await Future.doWhile(() =>
    events.length < num &&
        start + 1000 * timeout > DateTime.now().millisecondsSinceEpoch)
        .then((value) => events);

    if (events.length < num) {
      throw TimeoutException('Timeout reached while waiting for $num events');
    }
    return ret;
  }
}




Future updateData(Iterator<Event> iterator) async{

  while(iterator.moveNext()){


    var node = iterator.current.newNode;


              var temp = node!.path.toString().split(':');
              log('============>update');

              var type = '';
              temp[1] == Types.users ? type = Types.users : type = Types.devices;

              var type2 = '';
              temp[1] == Types.users ? type2 = Types.geigerScoreUser : type2 =
              Types.geigerScoreDevice;

              await updateMetrics(type, node);
              await updateScoreNode(type, type2);
              await updateRecommendationNode(type);

              await getDevices();
              await getEmployees();
              await crtAggregateScore();
              await updateScoreNode(Types.users, Types.geigerScoreAggregate);

  }


}

==> 2.0.5.tar.gz: lib/listener.dart <==
import 'dart:async';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'variables.dart';
import 'api.dart';
import 'dart:developer';
import 'function.dart';

class Event {
  final EventType _event;
  final Node? _old;
  final Node? _new;

  Event(this._event, this._old, this._new);

  EventType get type => _event;

  Node? get oldNode => _old;

  Node? get newNode => _new;

  @override
  String toString() {
    return '${type.toString()} ${oldNode.toString()}=>${newNode.toString()}';
  }
}

class SensorValueListener implements StorageListener {
  List<Event> events = <Event>[];

  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) async {
    Event e = Event(event, oldNode, newNode);

    log('============================got event ${e.newNode!.path}');

    if(newNode!.path.contains(':data:metrics') && newNode.name != 'metrics'){

      events.add(e);
      Storage.getValuePath.add(newNode.parentPath);
      await updateData(events.iterator);
      events.remove(e);
    }else if(newNode.path.contains(':data:GeigerScoreUser')){
      events.add(e);
      var str = await newNode.getValue('implementedRecommendations');

      log('geiger-indicator - new IR: '+ str!.getValue('en')!);
      log('geiger-indicator - old IR: '+ CoreValues.userImplementedRecommendations.toString());

      if(str.getValue('en') != CoreValues.userImplementedRecommendations.toString()) {
        generateImplementedReco(str.getValue('en'),Types.users);
        await updateScoreNode(Types.users, Types.geigerScoreUser);
      }
      events.remove(e);

    }else if(newNode.path.contains(':data:GeigerScoreDevice') ){

      events.add(e);
      var str = await newNode.getValue('implementedRecommendations');
      log('geiger-indicator - new IR: '+ str!.getValue('en')!);
      log('geiger-indicator - old IR: '+ CoreValues.userImplementedRecommendations.toString());

      if(str.getValue('en') != CoreValues.deviceImplementedRecommendations.toString()){
        generateImplementedReco(str.getValue('en'),Types.devices);
        await updateScoreNode(Types.devices, Types.geigerScoreDevice);
      }
      events.remove(e);
    }

    //if(newNode?.name != 'metrics') await updateData(newNode!);

  }

  Future<List<Event>> getNumEvents(int num, [int timeout = 2000]) async {
    int start = DateTime.now().millisecondsSinceEpoch;
    List<Event> ret = await Future.doWhile(() =>
    events.length < num &&
        start + 1000 * timeout > DateTime.now().millisecondsSinceEpoch)
        .then((value) => events);

    if (events.length < num) {
      throw TimeoutException('Timeout reached while waiting for $num events');
    }
    return ret;
  }
}




Future updateData(Iterator<Event> iterator) async{

  while(iterator.moveNext()){


    var node = iterator.current.newNode;


              var temp = node!.path.toString().split(':');
              log('============>update');

              var type = '';
              temp[1] == Types.users ? type = Types.users : type = Types.devices;

              var type2 = '';
              temp[1] == Types.users ? type2 = Types.geigerScoreUser : type2 =
              Types.geigerScoreDevice;

              await updateMetrics(type, node);
              await updateScoreNode(type, type2);
              await updateRecommendationNode(type);

              await getDevices();
              await getEmployees();
              await crtAggregateScore();
              await updateScoreNode(Types.users, Types.geigerScoreAggregate);

  }


}
